##random.area.R 

#R code for a randomization test that continuously selects sample units until the total area of the selected unit exceed given threshold

##Author
##Munemitsu Akasaka
##The University of Tokyo
##Japan
##mail2muu@gmail.com
##Last modified 25/05/11


#############################################################################################################
##Example of execution using sample data

##generating sample data
df.u <- data.frame(
	matrix(
			ifelse(runif(n=400, min=0, max=1) < 0.1, 1,0), ncol=20
	)
)
names.sp <- names(df.u) <- paste("sp", c(1:20), sep="")
df.u$area <- rpois(n=nrow(df.u), l=20)
thre <- sum(df.u$area)/2#half of the overall area is set as a threshold as an example

##example of execution using example data
out <- random.area.R(df.u = df.u, num.sam=10, cutlev= thre)#excution

###explanation of input data
##df.u: dataframe to use [data frame]
##num.sam: number of trial [numeric]
##cultlev: threshold area to select sampling units [numeric]

out#printing outputs

###explanation of output (list format)
##uniq: a matrix of species selected in each trail [matrix]
##numsp: number of (unique) species selected in each trial [matrix]
##ci: quantile (0, 2.5, 10, 20, 40, 50, 60, 80, 90, 97.5, and 100%) of number of (unique) species selected in overall trials [numeric vector]
#############################################################################################################

random.area.R <- function(df.u, num.sam, cutlev){

	#defining object "pool.seed"
	col.names <- c("area","numsp")
	pool.seed <- matrix(rep(NA, length(col.names)),nrow=1)
	colnames(pool.seed) <- col.names

	pool <- uniq.pool <- c()
	df.u.moto <- df.u
		
	for(i in 1:num.sam){#repeat
		gc()
		first.pick <- 0
		df.u <- df.u.moto #refresh df.u
		
		t.ar <- t.sam <- c()#initialize t.ar and t.sam

		repeat{#repeating selection of sampling unit (pond cluster) until total area of the selected unit reaches to the threshold (cutlev)
			s.num <- sample(1:nrow(df.u), 1, replace =F)
			
			pick.area <- df.u[s.num,"area"]

			if(first.pick !=0 | pick.area < cutlev){
				first.pick <- first.pick + 1
				t.sam <-  rbind(t.sam, df.u[s.num,])#sample one row and store it in t.sam
			
				t.ar <- sum(t.sam[["area"]], na.rm =T)#calculte total area selected and store it in t.ar

				df.u <- df.u[-1*s.num, ]#delete selected row of df.u
				
				if(t.ar > cutlev) break #finish selecting sampling unit if total area (t.ar) reaches defined threshold (cutlev) otherwise continue selection
			}
			
		 }

		t.uniqsps <- ifelse( apply(t.sam[names.sp], 2, sum) == 0,0,1)#store binary information on which species is included in selected units in t.uniqsps
		t.numsp <- sum(t.uniqsps)#store total number of (unique) species in selected units in t.numsp
		
		#store total area (t.ar) and total number of (unique) species (numsp) of a trail in t.pool
		t.pool <- pool.seed
		t.pool[,"area"] <- t.ar
		t.pool[,"numsp"] <- t.numsp

		pool <- rbind(pool, t.pool)
	 	uniq.pool <- rbind(uniq.pool, t.uniqsps)
	}
	
	y.ci <- quantile(pool[,"numsp"] , p=c(0,  0.025, 0.1, 0.2, 0.4, 0.5, 0.6, 0.8, 0.9, 0.975, 1), na.rm =T)#obtain quantile values of total number of species (= gamma diversity) and store them in y.ci
	return(list("uniq" = uniq.pool, "numsp"= pool,"ci"= y.ci))#
}

